@@ -31,7 +31,7 @@ class Roller:
             self.name,
             self.id,
             self.serial,
-            self.room.name,
+            self.room.name if self.room else "None",
             self.type,
             self.closed_percent,
             self.battery,
@@ -116,7 +116,7 @@ class Room:
         self.update_callbacks: List[Callable] = []
 
     def __str__(self):
-        """Returns string representation of roller."""
+        """Returns string representation of room."""
         return "Name: {} ID: {} Icon: {}".format(self.name, self.id[0:4], self.icon)
 
     def callback_subscribe(self, callback):
@@ -196,5 +196,33 @@ class Scene:
         self.name = None
 
     def __str__(self):
-        """Returns string representation of roller."""
+        """Returns string representation of scene."""
         return "Name: {} ID: {} Icon: {}".format(self.name, self.id[0:4], self.icon)
+
+
+class Timer:
+    """Representation of a Timer."""
+
+    def __init__(self, hub, timer_id):
+        """Init a new timer."""
+        self.hub = hub
+        self.id = timer_id
+        self.icon = None
+        self.name = None
+        self.state = None
+        self.hour = None
+        self.minute = None
+        self.days = None
+        self.entity = None
+
+    def __str__(self):
+        """Returns string representation of timer."""
+        return (
+            f"Name: {self.name} "
+            f"ID: {self.id[0:4]} "
+            f"Icon: {self.icon} "
+            f"State: {self.state} "
+            f"Time: {self.hour}:{self.minute} "
+            f"Days: {self.days:>07b} "
+            f'Entity: {self.entity.name if self.entity else "None"}'
+        )
