@@ -32,12 +32,11 @@ class Hub:
         self.topic = str.encode("Smart_Id1_y:")
         self.sequence = 4
         self.handshake = asyncio.Event()
-        self.event_update = asyncio.Event()
         self.response_task = None
         self.running = False
 
         self.name = "Acmeda Pulse WiFi Hub"
-        self.id = None
+        self.id = host
         self.host = host
         self.mac_address = None
         self.ip_address = None
@@ -49,10 +48,22 @@ class Hub:
         self.rollers = {}
         self.rooms = {}
         self.scenes = {}
+        self.timers = {}
 
         self.handshake.clear()
         self.update_callbacks: List[Callable] = []
 
+    def __str__(self):
+        """Returns string representation of the hub."""
+        return (
+            f"Name: {self.name} "
+            f"ID: {self.id} "
+            f"Host: {self.host} "
+            f"MAC: {self.mac_address} "
+            f"Firmware: {self.firmware_name} "
+            f"WiFi: {self.wifi_module} "
+        )
+
     def callback_subscribe(self, callback):
         """Add a callback for hub updates."""
         self.update_callbacks.append(callback)
@@ -236,56 +247,12 @@ class Hub:
         ptr += 2
         self.ip_address, ptr = utils.unpack_string(message, ptr)
 
-    def response_hubinfoend(self, message):
-        """Receive end of hub information."""
-        self.event_update.set()
-        self.notify_callback()
-
-    def response_roomlist(self, message):
-        """Receive room list."""
-        ptr = 12
-        room_count, ptr = utils.unpack_int(message, ptr, 1)
-        for _ in range(room_count):
-            _, ptr = utils.unpack_bytes(message, ptr, 2)
-            room_id, ptr = utils.unpack_string(message, ptr)
-            _, ptr = utils.unpack_bytes(message, ptr, 4)
-            icon, ptr = utils.unpack_int(message, ptr, 1)
-            _, ptr = utils.unpack_bytes(message, ptr, 2)
-            room_name, ptr = utils.unpack_string(message, ptr)
-            if room_id not in self.rooms:
-                self.rooms[room_id] = elements.Room(self, room_id)
-            self.rooms[room_id].icon = icon
-            self.rooms[room_id].name = room_name
-
-    def response_scenelist(self, message):
-        """Receive scene list."""
-        ptr = 12
-        scene_count, ptr = utils.unpack_int(message, ptr, 1)
-        ptr += 2
-        for _ in range(scene_count):
-            scene_id, ptr = utils.unpack_string(message, ptr)
-            _, ptr = utils.unpack_bytes(message, ptr, 4)
-            icon, ptr = utils.unpack_int(message, ptr, 1)
-            _, ptr = utils.unpack_bytes(message, ptr, 2)
-            scene_name, ptr = utils.unpack_string(message, ptr)
-            _, ptr = utils.unpack_bytes(message, ptr, 7)
-            _, ptr = utils.unpack_bytes(message, ptr)
-            _, ptr = utils.unpack_bytes(message, ptr, 2)
-            if scene_id not in self.scenes:
-                self.scenes[scene_id] = elements.Scene(self, scene_id)
-            self.scenes[scene_id].icon = icon
-            self.scenes[scene_id].name = scene_name
-
-    def response_timerlist(self, message):
-        """Receive timer list."""
-        pass
-
     def response_roller_updated(self, message):
         """Receive change of roller information."""
         ptr = 2  # sequence?
         ptr += 6
         ptr += 2  # unknown field
-        room_id, ptr = utils.unpack_string(message, ptr)
+        room_id, ptr = utils.unpack_bytes(message, ptr)
         ptr += 4  # unknown field
         roller_type, ptr = utils.unpack_int(message, ptr, 1)
         ptr += 2  # unknown field
@@ -317,6 +284,26 @@ class Hub:
         self.rollers[roller_id].notify_callback()
         self.notify_callback()
 
+    def response_discard(self, message):
+        """Discard response."""
+
+    def response_roomlist(self, message):
+        """Receive room list."""
+        ptr = 12
+        room_count, ptr = utils.unpack_int(message, ptr, 1)
+        for _ in range(room_count):
+            _, ptr = utils.unpack_bytes(message, ptr, 2)
+            room_id, ptr = utils.unpack_bytes(message, ptr)
+            _, ptr = utils.unpack_bytes(message, ptr, 4)
+            icon, ptr = utils.unpack_int(message, ptr, 1)
+            _, ptr = utils.unpack_bytes(message, ptr, 2)
+            room_name, ptr = utils.unpack_string(message, ptr)
+            if room_id not in self.rooms:
+                self.rooms[room_id] = elements.Room(self, room_id)
+            self.rooms[room_id].icon = icon
+            self.rooms[room_id].name = room_name
+        # self.notify_callback(ROOMS)
+
     def response_rollerlist(self, message):
         """Receive roller blind list."""
         ptr = 2  # sequence?
@@ -326,7 +313,7 @@ class Hub:
             ptr += 4  # unknown field
             roller_id, ptr = utils.unpack_int(message, ptr, 6)
             ptr += 2  # unknown field
-            room_id, ptr = utils.unpack_string(message, ptr)
+            room_id, ptr = utils.unpack_bytes(message, ptr)
             ptr += 4  # unknown field
             roller_type, ptr = utils.unpack_int(message, ptr, 1)
             ptr += 2  # unknown field
@@ -354,6 +341,87 @@ class Hub:
             self.rollers[roller_id].closed_percent = roller_percent
             self.rollers[roller_id].flags = roller_flags
             self.rollers[roller_id].notify_callback()
+        self.notify_callback()
+
+    def response_scenelist(self, message):
+        """Receive scene list."""
+        ptr = 0
+        _, ptr = utils.unpack_bytes(message, ptr, 12)
+        scene_count, ptr = utils.unpack_int(message, ptr, 1)
+        for _ in range(scene_count):
+            _, ptr = utils.unpack_bytes(message, ptr, 2)
+            scene_id, ptr = utils.unpack_bytes(message, ptr)
+            _, ptr = utils.unpack_bytes(message, ptr, 4)
+            icon, ptr = utils.unpack_int(message, ptr, 1)
+            _, ptr = utils.unpack_bytes(message, ptr, 2)
+            scene_name, ptr = utils.unpack_string(message, ptr)
+            _, ptr = utils.unpack_bytes(message, ptr, 5)
+            # Not sure what is being read next but it seems to be variable
+            while message[ptr : ptr + 2] == b"R\x02":
+                _, ptr = utils.unpack_bytes(message, ptr, 2)
+                _, ptr = utils.unpack_bytes(message, ptr)
+
+            if scene_id not in self.scenes:
+                self.scenes[scene_id] = elements.Scene(self, scene_id)
+            self.scenes[scene_id].icon = icon
+            self.scenes[scene_id].name = scene_name
+        _, ptr = utils.unpack_bytes(message, ptr, 2)
+        # self.notify_callback(SCENES)
+
+    def response_timerlist(self, message):
+        """Receive timer list."""
+        ptr = 0
+        _, ptr = utils.unpack_bytes(message, ptr, 12)
+        timer_count, ptr = utils.unpack_int(message, ptr, 1)
+        for _ in range(timer_count):
+            _, ptr = utils.unpack_bytes(message, ptr, 2)
+            timer_id, ptr = utils.unpack_bytes(message, ptr)
+            _, ptr = utils.unpack_bytes(message, ptr, 4)
+            icon, ptr = utils.unpack_int(message, ptr, 1)
+            _, ptr = utils.unpack_bytes(message, ptr, 2)
+            timer_name, ptr = utils.unpack_string(message, ptr)
+            _, ptr = utils.unpack_bytes(message, ptr, 4)  # '   !\x02\x01\x00'
+            state, ptr = utils.unpack_int(message, ptr, 1)
+            _, ptr = utils.unpack_bytes(message, ptr, 4)  # '   ;\x02\x01\x00'
+            hour, ptr = utils.unpack_int(message, ptr, 1)
+            _, ptr = utils.unpack_bytes(message, ptr, 4)  # '   <\x02\x01\x00'
+            minute, ptr = utils.unpack_int(message, ptr, 1)
+            _, ptr = utils.unpack_bytes(message, ptr, 4)  # '   "\x02\x04\x00'
+            days, ptr = utils.unpack_int(message, ptr, 1)
+            _, ptr = utils.unpack_bytes(message, ptr, 4)  # '\x00\x00\x00   ='
+            _, ptr = utils.unpack_bytes(message, ptr, 2)  # '\x02\x01'
+            timer_type, ptr = utils.unpack_bytes(message, ptr, 4)
+
+            entity = None
+            if timer_type == b"\x00\x01\x03\x01":  # Device Timer
+                _, ptr = utils.unpack_bytes(message, ptr, 8)
+                percent, ptr = utils.unpack_int(message, ptr, 1)
+                _, ptr = utils.unpack_bytes(message, ptr, 5)
+                roller_id, ptr = utils.unpack_int(message, ptr, 6)
+                if roller_id in self.rollers:
+                    entity = self.rollers[roller_id]
+            elif timer_type == b"\x00\x00\x10\x02":  # Scene Timer
+                scene_id, ptr = utils.unpack_bytes(message, ptr)
+                if scene_id in self.scenes:
+                    entity = self.scenes[scene_id]
+            else:
+                _LOGGER.error(
+                    f"{self.host}: Unexpected timer type received: "
+                    f"{binascii.hexlify(timer_type)}"
+                )
+                return
+
+            if timer_id not in self.timers:
+                self.timers[timer_id] = elements.Timer(self, timer_id)
+            self.timers[timer_id].icon = icon
+            self.timers[timer_id].name = timer_name
+            self.timers[timer_id].state = state
+            self.timers[timer_id].hour = hour
+            self.timers[timer_id].minute = minute
+            self.timers[timer_id].days = days
+            self.timers[timer_id].entity = entity
+        _, ptr = utils.unpack_bytes(message, ptr, 2)
+        # self.notify_callback(TIMERS)
 
     def response_authinfo(self, message):
         """Receive acmeda account information."""
@@ -401,13 +469,17 @@ class Hub:
 
     msgmap = {
         bytes.fromhex("1600"): Receiver("hub info", response_hubinfo),
-        bytes.fromhex("4101"): Receiver("hub info end", response_hubinfoend),
         bytes.fromhex("0101"): Receiver("room list", response_roomlist),
         bytes.fromhex("3301"): Receiver("scene list", response_scenelist),
         bytes.fromhex("2101"): Receiver("roller list", response_rollerlist),
+        bytes.fromhex("4101"): Receiver("timer list", response_timerlist),
         bytes.fromhex("0800"): Receiver("auth info", response_authinfo),
         bytes.fromhex("2301"): Receiver("position", response_position),
         bytes.fromhex("2501"): Receiver("roller info updated", response_roller_updated),
+        bytes.fromhex("4301"): Receiver("timer created", response_discard),
+        bytes.fromhex("4501"): Receiver("timer device updated", response_discard),
+        bytes.fromhex("4901"): Receiver("timer info updated", response_discard),
+        bytes.fromhex("4701"): Receiver("timer deleted", response_discard),
         bytes.fromhex("2b01"): Receiver("calibration", response_calibration),
         bytes.fromhex("0f00"): Receiver("discover response", response_discover),
     }
@@ -415,7 +487,6 @@ class Hub:
     def rec_ping(self, message):
         """Receive a ping from the hub."""
         _LOGGER.debug(f"{self.host}: Received hub ping response")
-        pass
 
     def rec_message(self, message):
         """Receive and decode a message from the hub."""
@@ -447,34 +518,8 @@ class Hub:
                 )
 
     respmap = {
-        const.RESPONSE_PING: Receiver("ping", rec_ping),
-        bytes.fromhex("01000091"): Receiver("scene list", rec_message),
-        bytes.fromhex("02000091"): Receiver("roller list", rec_message),
-        bytes.fromhex("03000091"): Receiver("acknowledge", rec_message),
-        bytes.fromhex("04000091"): Receiver(
-            "04000091", rec_message
-        ),  # Unknown from quentinsf
-        bytes.fromhex("23000091"): Receiver("hub info end", rec_message),
-        bytes.fromhex("28000091"): Receiver("discover", rec_message),
-        bytes.fromhex("34000091"): Receiver("moving", rec_message),
-        bytes.fromhex("42000091"): Receiver("unknown", rec_message),
-        bytes.fromhex("43000091"): Receiver("account info", rec_message),
-        bytes.fromhex("44000091"): Receiver("response move", rec_message),
-        const.RESPONSE_DISCOVER: Receiver("discover", rec_message),
-        bytes.fromhex("56000091"): Receiver("roller room change", rec_message),
-        bytes.fromhex("57000091"): Receiver("roller name change", rec_message),
-        bytes.fromhex("59000091"): Receiver("roller name change", rec_message),
-        bytes.fromhex("5b000091"): Receiver(
-            "room list", rec_message
-        ),  # after adding second room
-        bytes.fromhex("5F000091"): Receiver("hub info", rec_message),
-        bytes.fromhex("60000091"): Receiver("room list", rec_message),
-        bytes.fromhex("62000091"): Receiver(
-            "62000091", rec_message
-        ),  # Unknown from quentinsf
-        bytes.fromhex("63000091"): Receiver("hub info", rec_message),
-        bytes.fromhex("65000091"): Receiver("timer list", rec_message),
-        bytes.fromhex("6f000091"): Receiver("timer list", rec_message),
+        22: Receiver("ping", rec_ping),
+        145: Receiver("message", rec_message),
     }
 
     def response_parse(self, response):
@@ -486,25 +531,23 @@ class Hub:
             raise errors.InvalidResponseException
 
         try:
-            message_type = response[4:8]
-
-            if message_type[0] > 127:
-                message_type = response[5:9]
+            if response[4] > 127:
                 message = response[9:]
+                mtype = response[8]
             else:
                 message = response[8:]
+                mtype = response[7]
 
-            mtype = bytes(message_type)
             if mtype in Hub.respmap:
                 _LOGGER.debug(
-                    f"{self.host}: Received response: {binascii.hexlify(mtype)} "
+                    f"{self.host}: Received response: {mtype} "
                     f"{Hub.respmap[mtype].name} content: {message}"
                 )
                 Hub.respmap[mtype].execute(self, message)
             else:
                 _LOGGER.warning(
                     f"{self.host}: Received unknown response type: "
-                    f"{binascii.hexlify(mtype)}, "
+                    f"{mtype}, "
                     f"trying to decode anyway. Message: {binascii.hexlify(message)}"
                 )
                 self.rec_message(message)
@@ -534,7 +577,6 @@ class Hub:
 
     async def update(self):
         """Update all hub information (includes scenes, rooms, and rollers)."""
-        self.event_update.clear()
         await self.send_payload(
             const.COMMAND_GET_HUB_INFO,
             bytes.fromhex("F000"),
