@@ -26,7 +26,7 @@ class HubTransportBase(asyncio.Protocol):
 
     def connection_lost(self, exc):
         """Called when a connection is lost."""
-        _LOGGER.info("Socket closed")
+        _LOGGER.debug("Socket closed")
 
 
 class HubTransportUdp(HubTransportBase):
@@ -55,7 +55,7 @@ class HubTransportUdp(HubTransportBase):
     async def close(self):
         """Close the connection."""
         self.transport.close()
-        _LOGGER.info("UDP connection closed")
+        _LOGGER.debug("UDP connection closed")
 
     def send(self, buffer):
         """Abstraction of the underlying transport to send a buffer."""
@@ -102,21 +102,16 @@ class HubTransportTcp(HubTransportBase):
         self.transport = None
         self.protocol = None
         self.is_udp = False
+        self.connect_task = None
         super().__init__()
 
-    async def connect(self, host=None):
-        """Init connection."""
-        if host:
-            self.host = host
-
-        if self.writer:
-            _LOGGER.warn("Already connected.")
-            return
-
+    async def do_connection(self):
+        """Try and establish a TCP connection."""
         loop = asyncio.get_event_loop()
         self.reader = asyncio.StreamReader(loop=loop)
-        # super().__init__(self.reader)
         self.protocol = asyncio.StreamReaderProtocol(self.reader, loop=loop)
+
+        # The following blocks until a connection is made
         self.transport, _ = await loop.create_connection(
             lambda: self, self.host, self.port
         )
@@ -124,19 +119,42 @@ class HubTransportTcp(HubTransportBase):
             self.transport, self.protocol, self.reader, loop
         )
 
-    async def close(self):
-        """Close the connection."""
-        if not self.writer:
-            _LOGGER.error("Not connected")
+    async def connect(self, host=None):
+        """Init connection."""
+        if host:
+            self.host = host
+
+        if self.writer:
+            _LOGGER.warning(f"{self.host}: Already connected.")
             return
 
-        self.writer.close()
-        await self.writer.wait_closed()
-        self.writer = None
-        _LOGGER.info("TCP connection closed.")
+        if self.connect_task and not self.connect_task.done():
+            _LOGGER.warning(f"{self.host}: Already connecting.")
+        else:
+            self.connect_task = asyncio.create_task(self.do_connection())
+
+        await self.connect_task
+
+    async def close(self):
+        """Close the connection."""
+        if self.writer:
+            self.writer.close()
+            await self.writer.wait_closed()
+            self.writer = None
+            _LOGGER.debug(f"{self.host}: TCP buffer cleared.")
+
+        if self.transport:
+            self.transport.close()
+            _LOGGER.debug(f"{self.host}: TCP connection closed.")
+        elif self.connect_task and not self.connect_task.done():
+            self.connect_task.cancel()
+        else:
+            _LOGGER.warning(f"{self.host}: Not connected")
 
     def send(self, buffer):
         """Abstraction of the underlying transport to send a buffer."""
+        if not self.writer or self.writer.is_closing():
+            raise NotConnectedException
         self.writer.write(buffer)
 
     async def receive(self):
@@ -147,7 +165,6 @@ class HubTransportTcp(HubTransportBase):
 
     def data_received(self, data):
         """Callback when data has been received."""
-        _LOGGER.debug("TCP data received.")
         self.protocol.data_received(data)
 
     def connection_made(self, transport):
